#!python
# -*- coding: utf-8 -*-
__version__ = "$Revision: 1.13 $"
# $Source: /home/mechanoid/projects/py/cv/classifier-detector/lib/mymodule/RCS/data.py,v $
#
#       OS : GNU/Linux 4.10.3-1-ARCH 
# COMPILER : Python 3.6.0
#
#   AUTHOR : Evgeny S. Borisov
# 
#    http://www.mechanoid.kiev.ua
#  e-mail : nn@mechanoid.kiev.ua
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
import cv2
import numpy as np
import glob
import os


# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = 
class MLTrainData:

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def __init__(self, extractor):
        self.extractor=extractor
        self.X_train=[]
        self.y_train=[]
        self.X_test=[]
        self.y_test=[]
        self.X_val=[]
        self.y_val=[]
 
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def extract(self, f ):
        img = cv2.imread(f,cv2.IMREAD_GRAYSCALE) 
        h = self.extractor.extract(img)
        return [h]

  
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def extract_dir(self, d, part=1.0 ):
        if not os.path.exists(os.path.dirname(d)):
            raise Exception("[E] папка с примерами '%s' не найдена"%d)

        ff = glob.glob(d)
        p = max( min( int(len(ff)*part), len(ff) ) , 1 )
        idx = np.random.permutation(len(ff))  

        ff = [ ff[i] for i in idx[0:p] ] # формируем список файлов

        X = [] # результаты
        for f in ff:
            print("\t %s            "%f, end='\r')
            xf = self.extract(f)
            X.extend(xf)
            
        print("                                              ", end='\r')

        return np.array(X,dtype=np.float32)

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def reply(self,p,n):
        return np.vstack( ( np.ones( (p,1),dtype=np.int32) , -np.ones( (n,1), dtype=np.int32) )  )


    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def read_dirs(self, dir_pos=[["data/pgm/train/pos/*",1.0],], dir_neg=[["data/pgm/train/neg/*",1.0],] ):
        P = []
        for d,p in dir_pos:
            P.append( self.extract_dir(d,p) ) # позитивные примеры
        N = []
        for d,p in dir_neg:
            N.append( self.extract_dir(d,p) )# негативные примеры

        return np.vstack(P), np.vstack(N)


    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def make(self,
             dir_pos=[["data/pgm/train/pos/*",1.0],], # файлы позитивных примеров и доля используемых (случайно) примеров из общего количества 
             dir_neg=[["data/pgm/train/neg/*",1.0],], # файлы негативных примеро и доля используемых (случайно) примеров из общего количества 
             part = [ 0.0, 0.4 ] # доля наборов распределение validate, test, train (все остальные)
           ):

        print("[i] собираем данные ...")

        P,N = self.read_dirs( dir_pos, dir_neg )

        X = np.vstack( (P, N) )
        y = self.reply( P.shape[0], N.shape[0] )
  
        if ( part[1] > 0.0 ): # отрезаем тестовые примеры
            X, y, self.X_test, self.y_test = self.split( X, y, part[1] )

        if ( part[0] > 0.0 ):  # разделяем учебные и контрольные примеры
            self.X_train, self.y_train, self.X_val, self.y_val = self.split(X,y,part[0])
        else:
            self.X_train, self.y_train = X,y
            self.X_val, self.y_val = np.array([]), np.array([])


        print("\tразмер:",self.X_train.shape[1:])    
        print("\tучебных:",self.X_train.shape[0])    
        print("\tконтрольных:",self.X_val.shape[0])    
        print("\tтестовых:",self.X_test.shape[0])    


    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def split(self,X,y,part=0.1 ):
        s = X.shape[0] # количество примеров  
        p = max( min( int(s*part), s ) , 1 )
        
        idx = np.random.permutation(s) 

        X1 = np.array( [ X[i,:] for i in idx[0:p] ], dtype=np.float32 ) 
        y1 = np.array( [ y[i,:] for i in idx[0:p] ], dtype=np.int32 )

        X2 = np.array( [ X[i,:] for i in idx[p:] ], dtype=np.float32 )
        y2 = np.array( [ y[i,:] for i in idx[p:] ], dtype=np.int32 )

        return X2,y2,X1,y1

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def load(self,f="data/data.npz"):
        npzfile = np.load(f)

        self.X_train=npzfile["X_train"]
        self.y_train=npzfile["y_train"]

        self.X_test=npzfile["X_test"]
        self.y_test=npzfile["y_test"]

        self.X_val=npzfile["X_val"]
        self.y_val=npzfile["y_val"]
       
        
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def save(self,f="result/data.npz"):
        np.savez(f, X_train=self.X_train, y_train=self.y_train, X_val=self.X_val, y_val=self.y_val, X_test=self.X_test, y_test=self.y_test)


