#!python
# -*- coding: utf-8 -*-
__version__ = "$Revision: 1.3 $"
# $Source: /home/mechanoid/projects/py/cv/classifier-detector/lib/mymodule/RCS/detector.py,v $
#
#       OS : GNU/Linux 4.10.3-1-ARCH 
# COMPILER : Python 3.6.0
#
#   AUTHOR : Evgeny S. Borisov
# 
#    http://www.mechanoid.kiev.ua
#  e-mail : nn@mechanoid.kiev.ua
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
import cv2
import numpy as np
import os


# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = 
class WindowSlider:
    
    def __init__(self,window_size=(100,40),window_step=(10,5),scale_step=0.8):
        self.scale_step=scale_step
        self.window_size=window_size
        self.window_step=window_step 

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def scaler( self, img ):
       yield img
       while ( (img.shape[0]* self.scale_step >  self.window_size[1]) and (img.shape[1]* self.scale_step >  self.window_size[0]) ):
            img = cv2.resize(src=img, dsize=(0,0),fx=self.scale_step,fy=self.scale_step, interpolation=cv2.INTER_AREA)
            yield img


    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def slider( self, img ):
        xs,ys=self.window_step
        ww,wh=self.window_size
        h,w=img.shape

        for y in range(0, h-wh, ys):
            for x in range(0, w-ww, xs):
                yield ( x, y, img[ y:y+wh, x:x+ww ] )
 
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def run( self, img ): 
        i=1
        for img_scaled in self.scaler( img=img ):
            factor = img_scaled.shape[0]/img.shape[0]
            for (x,y,window) in self.slider( img=img_scaled ):
                self.proc_slide(window,(x,y),factor,i)
                i+=1

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def proc_slide( self, window,pos=(0,0), factor=1.0, index=0 ):
        cv2.imwrite("result/%04i-%.3f-%ix%i_%i-%i.png"%(index,factor,window.shape[1],window.shape[0],pos[0],pos[1]),window)
   



# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = 
class Detector(WindowSlider):

    def __init__(self, classificator, classificator_threshold=0.81, window_size=(100,40), window_step=(10,5), scale_step=0.8 ):
        self.classificator = classificator
        self.classificator_threshold = classificator_threshold
        self.boxes = []
        super().__init__(window_size, window_step, scale_step )


    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def search( self, img ):
        # img = cv2.imread(f,cv2.IMREAD_GRAYSCALE) # 100x40
        self.boxes = []
        self.run( img ) 
        self.clean_boxes() 

        return self.boxes

    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def proc_slide( self, window, pos=(0,0), factor=1.0, index=0 ):
    
        h,w = window.shape
        o = self.classificator.predict(window)
        #if o < -self.classificator_threshold:
        if o > self.classificator_threshold:
            x,y=pos
            rx, ry, rx2, ry2 = int(x * 1.0/factor), int(y * 1.0/factor), int((x+w) * 1.0/factor), int((y+h) * 1.0/factor)
            self.boxes.append([rx, ry, rx2, ry2, abs(o)])


    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def draw_boxes(self,img):
        img2 = cv2.cvtColor(img,cv2.COLOR_GRAY2RGB)
        font = cv2.FONT_HERSHEY_PLAIN

        for (x, y, x2, y2, score) in self.boxes:
            cv2.rectangle(img2, (int(x),int(y)),(int(x2), int(y2)),(0, 255, 0), 1)
            cv2.putText(img2, "%.3f" % score, (int(x),int(y)), font, 1, (0, 255, 0))
            #print(x, y, x2, y2, score)

        return img2


    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
    def clean_boxes(self):
        # - - - - - - - - - - - - - - - - - - 
        # расстояние между точками
        def dist(x,y):
            return np.sqrt(np.sum(np.square(np.array(center(x))-np.array(center(y)))) )

        # - - - - - - - - - - - - - - - - - - 
        # вычисляем точку-центр рамки
        def center(frame):
            (x, y, x2, y2, _ ) = frame
            return [ x+((x2-x)//2), y2+((y-y2)//2) ]

        # - - - - - - - - - - - - - - - - - - 
        # очистка списка найденных рамок
        def clean(X, i=0, max_dist = 35):
            # выбираем основной элемент X[i]
            j = i+1 # и, следующий за ним, элемент X[j]
            while(j<len(X)): # для всех элементов j (!= i)
                d = dist( X[i], X[j] ) # считаем расстояние между точками i,j
                if ( d < max_dist ): # i,j в одном кластере
                    if ( X[i][4] > X[j][4] ): # сравниваем рейтинг
                        del X[j]  # удаляем элемент j 
                    else:
                        del X[i] # удаляем основной элемент i
                        X = clean(X,i=i,max_dist=max_dist) # рекурсивно повторяем уже без элемента i
                        break
                else:
                    j+=1 # текущие точки i,j в разных кластерах, берём для проверки следующую точку j+1

            # найденный основной элемент i 
            # имеет наибольший рейтинг в своем кластере
            if ( i<(len(X)-1) ): # если есть ещё точки в других, относительно X[i], кластерах
                X = clean(X,i=i+1,max_dist=max_dist) # то выполняем проверку

            return X
        # - - - - - - - - - - - - - - - - - - 
        self.boxes = clean(self.boxes, max_dist = min(self.window_size) )


